<?php

declare(strict_types=1);

namespace Drupal\accessible_menu\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Accessible Menu settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'accessible_menu_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['accessible_menu.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $libraries = $this->config('accessible_menu.settings')->get('libraries');

    $form['library'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Library settings'),
      '#tree' => TRUE,
    ];

    $form['library']['accessible_menu'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Accessible Menu'),
      '#tree' => TRUE,
    ];

    $form['library']['accessible_menu']['installation'] = [
      '#type' => 'select',
      '#title' => $this->t('Installation method'),
      '#description' => $this->t('Choose if you want to use a local library or an external CDN to load accessible-menu.'),
      '#options' => [
        'cdn' => $this->t('External CDN'),
        'local' => $this->t('Local library'),
      ],
      '#default_value' => $libraries['accessible_menu']['installation'] ?? 'cdn',
      '#required' => TRUE,
    ];

    $form['library']['accessible_menu']['cdn'] = [
      '#type' => 'select',
      '#title' => $this->t('Select a CDN provdier'),
      '#description' => $this->t('Choose the CDN provider you want to use to load accessible-menu.'),
      '#options' => [
        'jsdelivr' => $this->t('jsDelivr'),
        'unpkg' => $this->t('UNPKG'),
      ],
      '#default_value' => $libraries['accessible_menu']['cdn'] ?? 'jsdelivr',
      '#states' => [
        'visible' => [
          'select[name="library[accessible_menu][installation]"]' => ['value' => 'cdn'],
        ],
      ],
    ];

    $form['library']['accessible_menu']['version'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Version'),
      '#description' => $this->t('Enter the version of the library you want to use. Use "latest" to always load the latest version.'),
      '#default_value' => $libraries['accessible_menu']['version'] ?? 'latest',
      '#states' => [
        'visible' => [
          'select[name="library[accessible_menu][installation]"]' => ['value' => 'cdn'],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $libraries = $form_state->getValue('library');

    foreach ($libraries as $key => $library) {
      $matches = [];

      if (!preg_match('/^(?<prefix>v)?(?<version>\d+\.\d+\.\d+(?:-(?:alpha|beta|rc)\.\d+)?)$/', $library['version'], $matches) && $library['version'] !== 'latest') {
        $form_state->setErrorByName(
          'accessible_menu[' . $key . '][version]',
          $this->t('The version must be a valid semver version or "latest".'),
        );
      }
      elseif (!empty($matches['prefix'])) {
        $form_state->setValue([$key, 'version'], $matches['version']);
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $libraries = $form_state->getValue('library');

    foreach ($libraries as $key => $library) {
      // Set the name.
      $libraries[$key]['name'] = 'Accessible Menu';

      $path = '/libraries/accessible-menu/dist/';
      // If CDN is selected, set the path to the CDN URL.
      if ($library['installation'] === 'cdn') {
        switch ($library['cdn']) {
          case 'jsdelivr':
            $path = 'https://cdn.jsdelivr.net/npm/accessible-menu@' . $library['version'] . '/dist/';
            break;

          case 'unpkg':
            $path = 'https://unpkg.com/accessible-menu@' . $library['version'] . '/dist/';
            break;
        }
      }

      $libraries[$key]['path'] = $path . 'accessible-menu.iife.js';
      $libraries[$key]['menu_types']['disclosure_menu'] = [
        'name' => 'Disclosure Menu',
        'constructor' => 'DisclosureMenu',
        'path' => $path . 'disclosure-menu.iife.js',
      ];
      $libraries[$key]['menu_types']['menubar'] = [
        'name' => 'Menubar',
        'constructor' => 'Menubar',
        'path' => $path . 'menubar.iife.js',
      ];
      $libraries[$key]['menu_types']['top_link_disclosure_menu'] = [
        'name' => 'Top Link Disclosure Menu',
        'constructor' => 'TopLinkDisclosureMenu',
        'path' => $path . 'top-link-disclosure-menu.iife.js',
      ];
      $libraries[$key]['menu_types']['treeview'] = [
        'name' => 'Treeview',
        'constructor' => 'Treeview',
        'path' => $path . 'treeview.iife.js',
      ];
    }

    $this->config('accessible_menu.settings')
      ->set('libraries', $libraries)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
